
import os
import json
import subprocess
import tempfile
import shutil
from utils.data_load import DataLoadUtil
from utils.reconstruction import ReconstructionUtil
from utils.pts import PtsUtil
class RenderUtil:

    @staticmethod
    def render_pts(cam_pose, scene_path, script_path, model_points_normals, voxel_threshold=0.005, filter_degree=75, nO_to_nL_pose=None, require_full_scene=False):
        
        nO_to_world_pose = DataLoadUtil.get_real_cam_O_from_cam_L(cam_pose, nO_to_nL_pose, scene_path=scene_path)

    
        with tempfile.TemporaryDirectory() as temp_dir:
            params = {
                "cam_pose": nO_to_world_pose.tolist(),
                "scene_path": scene_path
            }
            scene_info_path = os.path.join(scene_path, "scene_info.json")
            shutil.copy(scene_info_path, os.path.join(temp_dir, "scene_info.json"))
            params_data_path = os.path.join(temp_dir, "params.json")
            with open(params_data_path, 'w') as f:
                json.dump(params, f)
            result = subprocess.run([
                'blender', '-b', '-P', script_path, '--', temp_dir
            ], capture_output=True, text=True)
            if result.returncode != 0:
                print("Blender script failed:")
                print(result.stderr)
                return None
            path = os.path.join(temp_dir, "tmp")
            point_cloud = DataLoadUtil.get_target_point_cloud_world_from_path(path, binocular=True)
            cam_params = DataLoadUtil.load_cam_info(path, binocular=True)
            
            ''' TODO: old code: filter_points api is changed, need to update the code '''
            filtered_point_cloud = PtsUtil.filter_points(point_cloud, model_points_normals, cam_pose=cam_params["cam_to_world"], voxel_size=voxel_threshold, theta=filter_degree)
            full_scene_point_cloud = None
            if require_full_scene:
                depth_L, depth_R = DataLoadUtil.load_depth(path, cam_params['near_plane'], cam_params['far_plane'], binocular=True)
                point_cloud_L = DataLoadUtil.get_point_cloud(depth_L, cam_params['cam_intrinsic'], cam_params['cam_to_world'])['points_world']
                point_cloud_R = DataLoadUtil.get_point_cloud(depth_R, cam_params['cam_intrinsic'], cam_params['cam_to_world_R'])['points_world']
            
                point_cloud_L = PtsUtil.random_downsample_point_cloud(point_cloud_L, 65536)
                point_cloud_R = PtsUtil.random_downsample_point_cloud(point_cloud_R, 65536)
                full_scene_point_cloud = PtsUtil.get_overlapping_points(point_cloud_L, point_cloud_R)

            
            return filtered_point_cloud, full_scene_point_cloud